// Updates: 2004.07.29


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;


/**
 * A text field that record previous entries and allows to navigate between them
 * using the up and the down arrow keys. This actual version is specifically
 * made to work with the <code>RenameFiles</code> class, but can be easily updated
 * to work with any client (using an interface). The constructor gets as first
 * parameter the client that will be called (via the <code>tfChanged</code>
 * method) each time a key is pressed.
 * @author Michel Deriaz
 */
public class JTextFieldX extends JTextField implements ActionListener, KeyListener {
  private final int NB_ENTRIES = 25; // number of memorized previous entries
  private RenameFiles client;
  private String filename;
  private ArrayList cmds;
  private int cmdIndex;

  
  /**
   * Constructs a text field. The specified client (if not null) will be 
   * called (via the <code>tfChanged</code> method) each time a key is pressed.
   * @param client the client class
   * @param filename the file used to store previous entries
   * @param columns the size of the text field
   */
  public JTextFieldX(RenameFiles client, String filename, int columns) {
    super(columns);
    this.client = client;
    this.filename = filename;
    cmds = (ArrayList)readObject(filename);
    if (cmds == null) cmds = new ArrayList();
    cmdIndex = cmds.size();
    addKeyListener(this);
    addActionListener(this);
  }
  
  
  public void actionPerformed(ActionEvent e) {
    cmds.add(getText());
    cmdIndex = cmds.size();   
    if (cmds.size() > NB_ENTRIES) {
      cmds.remove(0);
      cmdIndex--;
    }
    writeObject(filename, cmds);
    selectAll();
  }  


  public void keyPressed(KeyEvent e) {
    if (e.getKeyCode() == KeyEvent.VK_UP) {
      if (cmdIndex > 0) setText((String)cmds.get(--cmdIndex));
      e.consume();
    }
    if (e.getKeyCode() == KeyEvent.VK_DOWN) {
      if (cmdIndex < cmds.size()-1) setText((String)cmds.get(++cmdIndex));
      e.consume();
    }
  }

  
  public void keyReleased(KeyEvent e) {
    if (client != null) client.tfChanged(this);
  }
  
  
  public void keyTyped(KeyEvent e) {}   
  

  private void writeObject(String filename, Object object) {
    try {
      FileOutputStream fos = new FileOutputStream(filename);
      ObjectOutputStream oos = new ObjectOutputStream(fos);
      oos.writeObject(object);
      oos.flush();
      fos.close();
      oos.close();
    }
    catch (IOException e) {
      System.out.println (e);
    }
  }  
  
  
  private Object readObject(String filename) {
    Object obj = null;
    try {
      FileInputStream fis = new FileInputStream(filename);
      ObjectInputStream ois = new ObjectInputStream(fis);
      obj = ois.readObject();
      fis.close();
      ois.close();
    }
    catch (IOException e) { // file doesn't exist
    }
    catch (ClassNotFoundException e) { 
      System.out.println(e);
    }
    return obj;
  }		   
}